# This file is part of node_annotator.
#
# node_annotator is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# node_annotator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with node_annotator.  If not, see <http://www.gnu.org/licenses/>.

# <pep8 compliant>

import bpy
import gpu
import gpu_extras
import mathutils
import os

from . import gpu_utils
from . import annotate_utils
from . import utils

class AnnotationPainter:
    draw_handler = None

    def __init__(self):
        AnnotationPainter.draw_handler = \
            bpy.types.SpaceNodeEditor.draw_handler_add(AnnotationPainter._draw_viewport,
                                                       (self,), "WINDOW", "POST_PIXEL")
        self.info_icon = None

    def shutdown(self):
        if AnnotationPainter.draw_handler is not None:
            bpy.types.SpaceNodeEditor.draw_handler_remove(AnnotationPainter.draw_handler, "WINDOW")
            AnnotationPainter.draw_handler = None

    def loadTexture(self, filename: str):
        directory = os.path.dirname(__file__)
        full_path = os.path.join(directory, "images", filename)
        image = bpy.data.images.load(full_path)
        texture = gpu.texture.from_image(image)
        bpy.data.images.remove(image)
        return texture

    def _draw_viewport(self):
        """ Callback function from blender to draw our hints into the viewport. Don't call by yourself! """
        context = bpy.context
        preferences = utils.get_preferences(context)
        if not preferences.show_description_icon:
            return
        
        if self.info_icon is None:
            self.info_icon = self.loadTexture("info_icon.png")
        
        icon_color = context.preferences.themes['Default'].user_interface.wcol_regular.text
        icon_alpha = context.preferences.themes['Default'].user_interface.icon_alpha
        # Although icon_color is already a Color object, we need to construct one again
        # to perform the color-space transformation, otherwise we get pure black.
        tint_color = mathutils.Color(icon_color).from_srgb_to_scene_linear()
        tint_color_alpha = (tint_color[0], tint_color[1], tint_color[2], icon_alpha)

        tree = bpy.context.space_data.edit_tree
        if tree is None:
            return
        view = bpy.context.region.view2d
        ui_scale = bpy.context.preferences.view.ui_scale
        node_header_size = 16 #  Didn't find any theme value, that represents the size of the node's header
        for n in tree.nodes:
            if n.type == 'GROUP' and getattr(n.node_tree, "description", "")!="":
                border = 2
                x1,y1 = view.view_to_region(n.location[0]*ui_scale,
                                            (n.location[1]-border)*ui_scale, clip=False)
                x2,y2 = view.view_to_region((n.location[0]+n.width-3*node_header_size)*ui_scale,
                                            (n.location[1]-node_header_size-border)*ui_scale, clip=False)
                icon_size = y1-y2

                oldState = gpu.state.blend_get()
                gpu.state.blend_set("ALPHA")
                gpu_utils.draw_tinted_texture_2d(self.info_icon, tint_color_alpha, (x2,y2), icon_size, icon_size)
                gpu.state.blend_set(oldState)
