# This file is part of node_annotator.
#
# node_annotator is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# node_annotator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with node_annotator.  If not, see <http://www.gnu.org/licenses/>.

# <pep8 compliant>


from cgitb import text
import bpy

from . import refactor_ops
from . import annotate_ops
from . import search_ops


class NodeAnnotatorPanel:
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = 'UI'
    bl_category = "Node"



class SearchResultsUIList(bpy.types.UIList):
    bl_idname = "NODE_UL_search_results"
    # The draw_item function is called for each item of the collection that is visible in the list.
    #   data is the RNA object containing the collection,
    #   item is the current drawn item of the collection,
    #   icon is the "computed" icon for the item (as an integer, because some objects like materials or textures
    #   have custom icons ID, which are not available as enum items).
    #   active_data is the RNA object containing the active property for the collection (i.e. integer pointing to the
    #   active item of the collection).
    #   active_propname is the name of the active property (use 'getattr(active_data, active_propname)').
    #   index is index of the current item in the collection.
    #   flt_flag is the result of the filtering process for this item.
    #   Note: as index and flt_flag are optional arguments, you do not have to use/declare them here if you don't
    #         need them.
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        # draw_item must handle the three layout types... Usually 'DEFAULT' and 'COMPACT' can share the same code.
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            # You should always start your row layout by a label (icon + text), or a non-embossed text field,
            # this will also make the row easily selectable in the list! The later also enables ctrl-click rename.
            # We use icon_value of label, as our given icon is an integer value, not an enum ID.
            # Note "data" names should never be translated!
            row = layout.row()
            row.label(text=item.path+"("+item.name+")", icon_value=icon)

            if not item.was_replaced:
                op = row.operator(refactor_ops.ReplaceCurrentNode.bl_idname, text="", icon="ARROW_LEFTRIGHT")
                op.node = item.name
                op.path = item.path
                op.index = index

            op = row.operator(search_ops.JumpToNode.bl_idname, text="", icon="ZOOM_SELECTED")
            op.node = item.name
            op.path = item.path
            op.index = index
            
        # 'GRID' layout type should be as compact as possible (typically a single icon!).
        elif self.layout_type == 'GRID':
            layout.alignment = 'CENTER'
            layout.label(text="", icon_value=icon)


class NodeAnnotatorMainPanel(NodeAnnotatorPanel, bpy.types.Panel):
    """ The main node annotator panel, shown in the Node tab of the graph view's side panel """
    bl_label = "Node Annotator"
    bl_idname = "NODE_PT_main_panel"

    @classmethod
    def poll(cls, context):
        return True
    
    def draw(self, context):
        space = context.space_data

        layout = self.layout
        if hasattr(space.edit_tree, "description"):
            layout.prop(space.edit_tree, "description")
        # Annotate section
        layout.label(text="Annotation")
        row = layout.row()
        row.operator(annotate_ops.CreateAnnotation.bl_idname)
        row.operator(annotate_ops.EditAnnotation.bl_idname)
        # Search section
        layout.separator()
        layout.label(text="Refactoring")
        refactor_op = context.window_manager.operator_properties_last(refactor_ops.ReplaceAllNodes.bl_idname)
        search_op = context.window_manager.operator_properties_last(search_ops.SearchSubGroup.bl_idname)
        layout.prop(search_op, "search_type")
        layout.prop(refactor_op, "new_group_type")
        layout.operator(search_ops.SearchSubGroup.bl_idname)

        search_results_are_for_tree = False
        if search_op is not None and space is not None and space.node_tree is not None:
            search_results_are_for_tree = (search_op.results_for == space.node_tree.name)
        if search_results_are_for_tree:
            layout.template_list(SearchResultsUIList.bl_idname, "", search_op, "results", search_op, "current_result")
        else:
            layout.label(text="Search results were for a different tree.")
        # Refactor section
        layout.operator(refactor_ops.ReplaceAllNodes.bl_idname)
