# This file is part of node_annotator.
#
# node_annotator is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# node_annotator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with node_annotator.  If not, see <http://www.gnu.org/licenses/>.

# <pep8 compliant>

import bpy

PATH_SEP = '|'

def list_all_node_groups(scene, context):
    """ This function needs to take the scene and the context,
        because it's used in an EnumProperty for the items callback.
        If we have a context, we use it to filter out incompatible groups."""
    tree = context.space_data.node_tree if context is not None else None
    tree_type = tree.type if tree is not None else None
    result = []
    for x in bpy.data.node_groups:
        if x.type == tree_type:
            entry = (x.name, x.name, "")
            result.append(entry)
    return result

def walk_path(path:list, path_string:str):
    path.clear()
    if path_string!='':
        split = path_string.split(PATH_SEP)
        for s in split:
            path.append(bpy.data.node_groups[s])

def escape_path_name(s:str):
    return s.replace(PATH_SEP, PATH_SEP+PATH_SEP)

def unescape_path_name(s:str):
    return s.replace(PATH_SEP+PATH_SEP, PATH_SEP)

def suitable_group_type(tree_type:str):
    match tree_type:
        case "GEOMETRY":
            return "GeometryNodeGroup"
        case "SHADER":
            return "ShaderNodeGroup"
        case "COMPOSITING":
            return "CompositorNodeGroup"
        case "TEXTURE":
            return "TextureNodeGroup"
        case _:
            return None

def replace_node(tree, node, new_type):
    new_group = tree.nodes.new(suitable_group_type(tree.type))
    new_group.color = node.color
    new_group.height = node.height
    new_group.label = node.label
    new_group.location = node.location
    new_group.mute = node.mute
    new_group.name = node.name
    new_group.node_tree = new_type
    new_group.parent = node.parent
    new_group.use_custom_color = node.use_custom_color
    new_group.width = node.width

    for node_in in node.inputs:
        if node_in.name in new_group.inputs:
            new_group_in = new_group.inputs[node_in.name]
            # Set the value
            if hasattr(node_in, "default_value")  and hasattr(new_group_in, "default_value"):
                new_group_in.default_value = node_in.default_value
            # And transfer the connections
            for connection in node_in.links:
                tree.links.new(connection.from_socket, new_group_in)
    for node_out in node.outputs:
        if node_out.name in new_group.outputs:
            for connection in node_out.links:
                tree.links.new(new_group.outputs[node_out.name], connection.to_socket)
    old_name = node.name
    tree.nodes.remove(node)
    new_group.name = old_name
    return new_group