# This file is part of node_annotator.
#
# node_annotator is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# node_annotator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with node_annotator.  If not, see <http://www.gnu.org/licenses/>.

# <pep8 compliant>

import bpy
from . import path_utils
from .search_ops import SearchSubGroup
from . import path_utils

class ReplaceNode(bpy.types.Operator):
    """Replaces a node with a new version of the same type"""
    bl_idname = "node_annotator.replace_node"
    bl_label = "Replace"
    bl_options = {'REGISTER', 'UNDO'}

    # The node, to replace
    node: bpy.props.StringProperty(name="Node")
    # The path to the parent group, where the node is instanced in.
    path: bpy.props.StringProperty(name="Path")
    # The new group, to change the node to
    new_group_type: bpy.props.EnumProperty(name="Replacement type",
                                           description="Select the type exchange the selected node with.",
                                           items=path_utils.list_all_node_groups)

    def __init__(self):
        pass
    
    def do_execute(self, context):
        space = context.space_data 
        path=[]
        path_utils.walk_path(path, self.path)
        tree = path[-1] if len(path)>0 else space.node_tree
        node = tree.nodes[self.node]
        new_type = bpy.data.node_groups[self.new_group_type]
        new_group = path_utils.replace_node(tree, node, new_type)
        return tree, new_group


class ReplaceCurrentNode(ReplaceNode):
    """Replaces a node with a new version of the same type"""
    bl_idname = "node_annotator.replace_current_node"
    bl_label = "Replace"
    bl_options = {'REGISTER', 'UNDO'}

    # The result index to make it disabled, when refactoring.
    index: bpy.props.IntProperty(name="Index")

    def __init__(self):
        pass

    @classmethod
    def poll(cls, context):
        tree = context.space_data.node_tree
        type_ok = path_utils.suitable_group_type(tree.type) is not None
        return type_ok


    def execute(self, context):
        replace_op_props = context.window_manager.operator_properties_last(ReplaceAllNodes.bl_idname)
        self.new_group_type = replace_op_props.new_group_type
        search_op_props = context.window_manager.operator_properties_last(SearchSubGroup.bl_idname)
        current_result_data = search_op_props.results[self.index]
        current_result_data.was_replaced = True
        tree, new_group = super().do_execute(context)
        current_result_data.name = new_group.name
        new_group.select = True
        tree.nodes.active = new_group
        return {'FINISHED'}


class ReplaceAllNodes(ReplaceNode):
    """Replaces all node with a new version of the same type"""
    bl_idname = "node_annotator.replace_all_nodes"
    bl_label = "Replace all"
    bl_options = {'REGISTER', 'UNDO'}

    def __init__(self):
        pass

    @classmethod
    def poll(cls, context):
        tree = context.space_data.node_tree
        search_op_props = context.window_manager.operator_properties_last(SearchSubGroup.bl_idname)
        type_ok = path_utils.suitable_group_type(tree.type) is not None
        return tree is not None and tree.name==search_op_props.results_for and type_ok

    def execute(self, context):
        search_op_props = context.window_manager.operator_properties_last(SearchSubGroup.bl_idname)
        for current_result_data in search_op_props.results:
            if not current_result_data.was_replaced:
                self.node = current_result_data.name
                self.path = current_result_data.path
                tree, new_group = super().do_execute(context)
                current_result_data.was_replaced = True
                current_result_data.name = new_group.name
        return {'FINISHED'}
