# This file is part of node_annotator.
#
# node_annotator is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# node_annotator is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with node_annotator.  If not, see <http://www.gnu.org/licenses/>.

# <pep8 compliant>

import bpy

from . import path_utils

class SearchResultsProperties(bpy.types.PropertyGroup):
    name: bpy.props.StringProperty()
    path: bpy.props.StringProperty()
    was_replaced: bpy.props.BoolProperty(default=False)

class SearchSubGroup(bpy.types.Operator):
    """Create a new annotation in the given node graph"""
    bl_idname = "node_annotator.search_sub_group"
    bl_label = "Search subgroup"
    bl_options = {'REGISTER', 'UNDO'}
    test_feedback = None

    search_type: bpy.props.EnumProperty(name="Search type",
                                        description="Select the type to search.",
                                        items=path_utils.list_all_node_groups)
    current_result: bpy.props.IntProperty(name="current_result")
    results: bpy.props.CollectionProperty(type=SearchResultsProperties)
    results_for: bpy.props.StringProperty(name="Node tree",
                                          description="The tree the results were found in.")

    def __init__(self):
        pass

    @classmethod
    def poll(cls, context):
        tree = context.space_data.node_tree
        return tree is not None

    def execute(self, context):
        self.results.clear()
        space = context.space_data
        self.results_for = space.node_tree.name # Remember the root tree
        current_path = [path_utils.escape_path_name(x.node_tree.name) for x in space.path]
        if len(current_path)>0:
            current_path.pop(0)
        traversed_trees = set()
        # Search from currently edited group
        self.traverse(space.edit_tree, current_path, traversed_trees)
        return {'FINISHED'}

    def traverse(self, tree, current_path, traversed_trees):
        for n in tree.nodes:
            if hasattr(n, 'node_tree'):
                if n.node_tree.name==self.search_type:
                    item = self.results.add()
                    item.name = n.name
                    path = path_utils.PATH_SEP.join(current_path)
                    item.path = path
                    if SearchSubGroup.test_feedback is not None:
                        SearchSubGroup.test_feedback(n.name, path)
                if n.node_tree not in traversed_trees:
                    traversed_trees.add(n.node_tree)
                    current_path.append(path_utils.escape_path_name(n.node_tree.name))
                    self.traverse(n.node_tree, current_path, traversed_trees)
                    current_path.pop()


class JumpToNode(bpy.types.Operator):
    """Jumps to a specified node"""
    bl_idname = "node_annotator.jump_to_node"
    bl_label = "Search subgroup"
    bl_options = {'REGISTER', 'UNDO'}

    # The node, that was found.
    node: bpy.props.StringProperty(name="Node")
    # The path to the parent group, where the node is instanced in.
    path: bpy.props.StringProperty(name="Path")
    # The result index to make it current, when jumping there.
    index: bpy.props.IntProperty(name="Index")

    def __init__(self):
        pass

    def find_window_region(self, area):
        for r in area.regions:
            if r.type=='WINDOW':
                return r
        return None

    def execute(self, context):
        space = context.space_data
        path_utils.walk_path(space.path, self.path)
        path_len = len(space.path)
        if path_len>0:
            bpy.ops.node.select_all(action='DESELECT')
            tree = space.path[path_len-1].node_tree
            node = tree.nodes[self.node]
            node.select = True
            tree.nodes.active = node

            search_op_props = context.window_manager.operator_properties_last(SearchSubGroup.bl_idname)
            search_op_props.current_result = self.index
            
            region = self.find_window_region(context.area)
            with context.temp_override(region=region):
                return bpy.ops.node.view_selected('INVOKE_DEFAULT')

        return {'FINISHED'}
